@@ -278,6 +278,7 @@ class Install(install_misc.InstallBase):
                         if sb_var_file.read(1) == b"\x01":
                             keep.add('grub-efi-amd64-signed')
                             keep.add('shim-signed')
+                            keep.add('linux-signed-generic')
             else:
                 keep.add('grub')
                 keep.add('grub-pc')
@@ -525,16 +526,22 @@ class Install(install_misc.InstallBase):
             prefix = os.path.basename(kernel).split('-', 1)[0]
             release = os.uname()[2]
             target_kernel = os.path.join(bootdir, '%s-%s' % (prefix, release))
-            osextras.unlink_force(target_kernel)
-            install_misc.copy_file(self.db, kernel, target_kernel, md5_check)
-            os.lchown(target_kernel, 0, 0)
-            os.chmod(target_kernel, 0o644)
-            st = os.lstat(kernel)
-            try:
-                os.utime(target_kernel, (st.st_atime, st.st_mtime))
-            except Exception:
-                # We can live with timestamps being wrong.
-                pass
+            copies = [(kernel, target_kernel)]
+            if os.path.exists("%s.efi.signed" % kernel):
+                copies.append(
+                    ("%s.efi.signed" % kernel,
+                     "%s.efi.signed" % target_kernel))
+            for source, target in copies:
+                osextras.unlink_force(target)
+                install_misc.copy_file(self.db, source, target, md5_check)
+                os.lchown(target, 0, 0)
+                os.chmod(target, 0o644)
+                st = os.lstat(source)
+                try:
+                    os.utime(target, (st.st_atime, st.st_mtime))
+                except Exception:
+                    # We can live with timestamps being wrong.
+                    pass
 
         os.umask(old_umask)
 
